@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -18,14 +18,14 @@
 
 #include "config.h"
 
-#include	<stdio.h>
-#include	<string.h>
-#include	<time.h>
-#include	<ctype.h>
+#include <stdio.h>
+#include <string.h>
+#include <time.h>
+#include <ctype.h>
 
-#include	"sndfile.h"
-#include	"sfendian.h"
-#include	"common.h"
+#include "sndfile.h"
+#include "sfendian.h"
+#include "common.h"
 
 /*------------------------------------------------------------------------------
  * Macros to handle big/little endian issues.
@@ -264,15 +264,13 @@ aiff_open	(SF_PRIVATE *psf)
 				error = SFE_DWVW_BAD_BITWIDTH ;
 				break ;
 
-		/*-
 		case SF_FORMAT_IMA_ADPCM :
-				/+*
+				/*
 				**	IMA ADPCM encoded AIFF files always have a block length
 				**	of 34 which decodes to 64 samples.
-				*+/
+				*/
 				error = aiff_ima_init (psf, AIFC_IMA4_BLOCK_LEN, AIFC_IMA4_SAMPLES_PER_BLOCK) ;
 				break ;
-		-*/
 		/* Lite remove end */
 
 		case SF_FORMAT_GSM610 :
@@ -327,7 +325,7 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 					if (psf->fileoffset > 0 && psf->filelength > FORMsize + 8)
 					{	/* Set file length. */
-						psf->filelength = FORMsize + 8  ;
+						psf->filelength = FORMsize + 8 ;
 						psf_log_printf (psf, "FORM : %u\n", FORMsize) ;
 						}
 					else if (FORMsize != psf->filelength - 2 * SIGNED_SIZEOF (dword))
@@ -795,12 +793,10 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 				break ;
 
 
-		/*
 		case ima4_MARKER :
 				psf->endian = SF_ENDIAN_BIG ;
 				psf->sf.format = (SF_FORMAT_AIFF | SF_FORMAT_IMA_ADPCM) ;
 				break ;
-		*/
 
 		default :
 			psf_log_printf (psf, "AIFC : Unimplemented format : %M\n", comm_fmt->encoding) ;
@@ -901,14 +897,14 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 				psf->endian = SF_ENDIAN_BIG ;
 				comm_type = AIFC_MARKER ;
 				comm_size = SIZEOF_AIFC_COMM ;
-				comm_encoding = FL32_MARKER ; /* Use 'FL32' because its easier to read. */
+				comm_encoding = FL32_MARKER ;	/* Use 'FL32' because its easier to read. */
 				break ;
 
 		case SF_FORMAT_DOUBLE :					/* Big endian double precision floating point. */
 				psf->endian = SF_ENDIAN_BIG ;
 				comm_type = AIFC_MARKER ;
 				comm_size = SIZEOF_AIFC_COMM ;
-				comm_encoding = FL64_MARKER ; /* Use 'FL64' because its easier to read. */
+				comm_encoding = FL64_MARKER ;	/* Use 'FL64' because its easier to read. */
 				break ;
 
 		case SF_FORMAT_ULAW :
@@ -972,6 +968,17 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 				bit_width = 16 ;
 				break ;
 
+		case SF_FORMAT_IMA_ADPCM :
+				psf->endian = SF_ENDIAN_BIG ;
+				comm_type = AIFC_MARKER ;
+				comm_size = SIZEOF_AIFC_COMM ;
+				comm_encoding = ima4_MARKER ;
+
+				/* Override standard value here.*/
+				bit_width = 16 ;
+				comm_frames = psf->sf.frames / AIFC_IMA4_SAMPLES_PER_BLOCK ;
+				break ;
+
 		default : return SFE_BAD_OPEN_FORMAT ;
 		} ;
 
