@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -297,7 +297,7 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 					{	c = *strptr++ ;
 						LOG_PUTCHAR (psf, c) ;
 						} ;
-							break ;
+					break ;
 
 			default :
 					LOG_PUTCHAR (psf, '*') ;
@@ -323,11 +323,11 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 void
 psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 {	va_list	argptr ;
-	int  maxlen ;
-	char *start ;
+	int		maxlen ;
+	char	*start ;
 
-	start  = (char*) psf->header + strlen ((char*) psf->header) ;
-	maxlen = sizeof (psf->header) - strlen ((char*) psf->header) ;
+	start	= (char*) psf->header + strlen ((char*) psf->header) ;
+	maxlen	= sizeof (psf->header) - strlen ((char*) psf->header) ;
 
 	va_start (argptr, format) ;
 	LSF_VSNPRINTF (start, maxlen, format, argptr) ;
@@ -384,24 +384,24 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 */
 
 #define	PUT_BYTE(psf,x)		if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 1)	\
-							{	(psf)->header [(psf)->headindex++] = (x) ;   }
+							{ (psf)->header [(psf)->headindex++] = (x) ; }
 
 #if (CPU_IS_BIG_ENDIAN == 1)
 #define	PUT_MARKER(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 24) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
-								(psf)->header [(psf)->headindex++] = (x) ;   }
+								(psf)->header [(psf)->headindex++] = (x) ;	}
 
 #elif (CPU_IS_LITTLE_ENDIAN == 1)
 #define	PUT_MARKER(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
-								(psf)->header [(psf)->headindex++] = ((x) >>  8) ;		\
+								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
-								(psf)->header [(psf)->headindex++] = ((x) >> 24) ;   }
+								(psf)->header [(psf)->headindex++] = ((x) >> 24) ;	}
 
 #else
-#       error "Cannot determine endian-ness of processor."
+#	error "Cannot determine endian-ness of processor."
 #endif
 
 
@@ -420,7 +420,7 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 
 #define	PUT_LE_3BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 3)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
-								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
+								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;	}
 
 #define	PUT_BE_INT(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
@@ -431,7 +431,7 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 
 #define	PUT_LE_INT(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
-								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
+								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 24) ;	}
 
@@ -448,15 +448,15 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 
 #define	PUT_LE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
-								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
+								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 24) ;		\
 								(psf)->header [(psf)->headindex++] = 0 ;				\
-								(psf)->header [(psf)->headindex++] = 0 ;   				\
+								(psf)->header [(psf)->headindex++] = 0 ;				\
 								(psf)->header [(psf)->headindex++] = 0 ;				\
 								(psf)->header [(psf)->headindex++] = 0 ;	}
 
-#elif  (SIZEOF_SF_COUNT_T == 8)
+#elif (SIZEOF_SF_COUNT_T == 8)
 #define	PUT_BE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 56) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 48) ;		\
@@ -469,11 +469,11 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 
 #define	PUT_LE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
-								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
+								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 24) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 32) ;		\
-								(psf)->header [(psf)->headindex++] = ((x) >> 40) ;   	\
+								(psf)->header [(psf)->headindex++] = ((x) >> 40) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 48) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 56) ;	}
 #else
@@ -490,7 +490,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 	double			doubledata ;
 	void			*bindata ;
 	size_t			size ;
-	char    		c, *strptr ;
+	char			c, *strptr ;
 	int				count = 0, trunc_8to4 ;
 
 	trunc_8to4 = SF_FALSE ;
@@ -605,8 +605,8 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 
 			case 's' :
 					strptr = va_arg (argptr, char *) ;
-					size   = strlen (strptr) + 1 ;
-					size  += (size & 1) ;
+					size = strlen (strptr) + 1 ;
+					size += (size & 1) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 					{	PUT_BE_INT (psf, size) ;
 						}
@@ -620,23 +620,23 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					break ;
 
 			case 'S' :
-					strptr = va_arg (argptr, char *) ;
-					size   = strlen (strptr) + 1 ;
+					strptr	= va_arg (argptr, char *) ;
+					size	= strlen (strptr) + 1 ;
 					memcpy (&(psf->header [psf->headindex]), strptr, size) ;
 					psf->headindex += size ;
 					count += size ;
 					break ;
 
 			case 'b' :
-					bindata = va_arg (argptr, void *) ;
-					size    = va_arg (argptr, size_t) ;
+					bindata	= va_arg (argptr, void *) ;
+					size	= va_arg (argptr, size_t) ;
 					memcpy (&(psf->header [psf->headindex]), bindata, size) ;
 					psf->headindex += size ;
 					count += size ;
 					break ;
 
 			case 'z' :
-					size    = va_arg (argptr, size_t) ;
+					size = va_arg (argptr, size_t) ;
 					count += size ;
 					while (size)
 					{	psf->header [psf->headindex] = 0 ;
@@ -682,45 +682,45 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 */
 
 #if (CPU_IS_BIG_ENDIAN == 1)
-#define	GET_MARKER(ptr)	( ((ptr) [0] << 24) | ((ptr) [1] << 16) |	\
-						  ((ptr) [2] <<  8) | ((ptr) [3]) )
+#define	GET_MARKER(ptr)	(	((ptr) [0] << 24)	| ((ptr) [1] << 16) |	\
+							((ptr) [2] << 8)	| ((ptr) [3]) )
 
 #elif (CPU_IS_LITTLE_ENDIAN == 1)
 #define	GET_MARKER(ptr)	(	((ptr) [0])			| ((ptr) [1] << 8) |	\
-						  ((ptr) [2] << 16) | ((ptr) [3] << 24) )
+							((ptr) [2] << 16)	| ((ptr) [3] << 24) )
 
 #else
-#       error "Cannot determine endian-ness of processor."
+#	error "Cannot determine endian-ness of processor."
 #endif
 
-#define	GET_LE_SHORT(ptr)	( ((ptr) [1] <<  8) | ((ptr) [0]) )
-#define	GET_BE_SHORT(ptr)	( ((ptr) [0] <<  8) | ((ptr) [1]) )
+#define	GET_LE_SHORT(ptr)	( ((ptr) [1] << 8) | ((ptr) [0]) )
+#define	GET_BE_SHORT(ptr)	( ((ptr) [0] << 8) | ((ptr) [1]) )
 
 #define	GET_LE_3BYTE(ptr)	( 	((ptr) [2] << 16) | ((ptr) [1] << 8) | ((ptr) [0]) )
 #define	GET_BE_3BYTE(ptr)	( 	((ptr) [0] << 16) | ((ptr) [1] << 8) | ((ptr) [2]) )
 
-#define	GET_LE_INT(ptr)		( 	((ptr) [3] << 24) | ((ptr) [2] << 16) |	\
-								((ptr) [1] <<  8) | ((ptr) [0]) )
+#define	GET_LE_INT(ptr)		( 	((ptr) [3] << 24)	| ((ptr) [2] << 16) |	\
+								((ptr) [1] << 8)	| ((ptr) [0]) )
 
-#define	GET_BE_INT(ptr)		( 	((ptr) [0] << 24) | ((ptr) [1] << 16) |	\
-							 	((ptr) [2] <<  8) | ((ptr) [3]) )
+#define	GET_BE_INT(ptr)		( 	((ptr) [0] << 24)	| ((ptr) [1] << 16) |	\
+							 	((ptr) [2] << 8)	| ((ptr) [3]) )
 
 #if (SIZEOF_LONG == 4)
-#define	GET_LE_8BYTE(ptr)	( 	((ptr) [3] << 24) | ((ptr) [2] << 16) |	\
-							 	((ptr) [1] <<  8) | ((ptr) [0]) )
+#define	GET_LE_8BYTE(ptr)	( 	((ptr) [3] << 24)	| ((ptr) [2] << 16) |	\
+							 	((ptr) [1] << 8)	| ((ptr) [0]) )
 
-#define	GET_BE_8BYTE(ptr)	( 	((ptr) [4] << 24) | ((ptr) [5] << 16) |	\
-								((ptr) [6] <<  8) | ((ptr) [7]) )
+#define	GET_BE_8BYTE(ptr)	( 	((ptr) [4] << 24)	| ((ptr) [5] << 16) |	\
+								((ptr) [6] << 8)	| ((ptr) [7]) )
 #else
-#define	GET_LE_8BYTE(ptr)	( 	((ptr) [7] << 56L) | ((ptr) [6] << 48L) |	\
-							 	((ptr) [5] << 40L) | ((ptr) [4] << 32L) |	\
-							 	((ptr) [3] << 24L) | ((ptr) [2] << 16L) |	\
-							 	((ptr) [1] <<  8L) | ((ptr) [0] ))
+#define	GET_LE_8BYTE(ptr)	( 	((ptr) [7] << 56L)	| ((ptr) [6] << 48L) |	\
+							 	((ptr) [5] << 40L)	| ((ptr) [4] << 32L) |	\
+							 	((ptr) [3] << 24L)	| ((ptr) [2] << 16L) |	\
+							 	((ptr) [1] << 8L)	| ((ptr) [0] ))
 
-#define	GET_BE_8BYTE(ptr)	( 	((ptr) [0] << 56L) | ((ptr) [1] << 48L) |	\
-							 	((ptr) [2] << 40L) | ((ptr) [3] << 32L) |	\
-							 	((ptr) [4] << 24L) | ((ptr) [5] << 16L) |	\
-							 	((ptr) [6] <<  8L) | ((ptr) [7] ))
+#define	GET_BE_8BYTE(ptr)	( 	((ptr) [0] << 56L)	| ((ptr) [1] << 48L) |	\
+							 	((ptr) [2] << 40L)	| ((ptr) [3] << 32L) |	\
+							 	((ptr) [4] << 24L)	| ((ptr) [5] << 16L) |	\
+							 	((ptr) [6] << 8L)	| ((ptr) [7] ))
 
 #endif
 
@@ -735,9 +735,9 @@ header_read (SF_PRIVATE *psf, void *ptr, int bytes)
 		bytes = sizeof (psf->header) - psf->headindex ;
 		if (bytes <= 0)
 			return 0;
-		printf ("\nR sizeof (header): %u    headindex: %4u    bytes: %d\n", sizeof (psf->header), psf->headindex, bytes) ;
+		printf ("\nR sizeof (header): %d    headindex: %4u    bytes: %d\n", SIGNED_SIZEOF (psf->header), psf->headindex, bytes) ;
 		} ;
-		
+
 
 	if (psf->headindex + bytes > psf->headend)
 	{	count = psf_fread (psf->header + psf->headend, 1, bytes - (psf->headend - psf->headindex), psf) ;
@@ -759,16 +759,14 @@ header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 {
 	switch (whence)
 	{	case SEEK_SET :
-			if (position > sizeof (psf->header))
+			if (position > SIGNED_SIZEOF (psf->header))
 			{	/* Too much header to cache so just seek instead. */
 				psf_fseek (psf, position, whence) ;
-//printf ("A sizeof (header): %u    headindex: %4u\n", sizeof (psf->header), psf->headindex) ;
 				return ;
 				} ;
 			if (position > psf->headend)
 				psf->headend += psf_fread (psf->header + psf->headend, 1, position - psf->headend, psf) ;
 			psf->headindex = position ;
-//printf ("B sizeof (header): %u    headindex: %4u\n", sizeof (psf->header), psf->headindex) ;
 			break ;
 
 		case SEEK_CUR :
@@ -781,10 +779,9 @@ header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 				} ;
 
 			if (psf->headindex + position > SIGNED_SIZEOF (psf->header))
-			{	psf->headend += psf_fread (psf->header + psf->headend, 1, SIGNED_SIZEOF (psf->header) - psf->headend, psf) ;
-				psf->headindex = psf->headend = SIGNED_SIZEOF (psf->header) ;
-
-				psf_fseek (psf, position - (SIGNED_SIZEOF (psf->header) - psf->headend), SEEK_CUR) ;
+			{	/* Need to jump this without caching it. */
+				psf->headindex = psf->headend ;
+				psf_fseek (psf, position, SEEK_CUR) ;
 				break ;
 				} ;
 
@@ -833,7 +830,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 	unsigned char	*ucptr, sixteen_bytes [16] ;
 	unsigned int 	*intptr, intdata ;
 	unsigned short	*shortptr ;
-	char    		*charptr ;
+	char			*charptr ;
 	float			*floatptr ;
 	double			*doubleptr ;
 	size_t			size ;
@@ -1102,7 +1099,7 @@ psf_memset (void *s, int c, sf_count_t len)
 	return s ;
 } /* psf_memset */
 
-void  psf_get_date_str (char *str, int maxlen)
+void psf_get_date_str (char *str, int maxlen)
 {	time_t		current ;
 	struct tm	timedata, *tmptr ;
 
@@ -1125,7 +1122,7 @@ void  psf_get_date_str (char *str, int maxlen)
 			timedata.tm_hour, timedata.tm_min, timedata.tm_sec) ;
 	else
 		LSF_SNPRINTF (str, maxlen, "Unknown date") ;
-		
+
 	return ;
 } /* psf_get_date_str */
 
@@ -1177,7 +1174,7 @@ u_bitwidth_to_subformat (int bits)
 #endif /* PSF_LOG_PRINTF_ONLY */
 /*
 ** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
+** The arch-tag line is a file identity tag for the GNU Arch
 ** revision control system.
 **
 ** arch-tag: 33e9795e-f717-461a-9feb-65d083a56395
