@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -26,14 +26,17 @@
 #include	"float_cast.h"
 #include	"common.h"
 
-typedef struct
-{	int				channels, blocksize, samplesperblock, blocks ;
+typedef struct IMA_ADPCM_PRIVATE_tag
+{	int 			(*decode_block) (SF_PRIVATE *psf, struct IMA_ADPCM_PRIVATE_tag *pima) ;
+	int 			(*encode_block) (SF_PRIVATE *psf, struct IMA_ADPCM_PRIVATE_tag *pima) ;
+
+	int				channels, blocksize, samplesperblock, blocks ;
 	int				blockcount, samplecount ;
 	int				previous [2] ;
 	int				stepindx [2] ;
 	unsigned char	*block ;
 	short			*samples ;
-	unsigned char	data	[4] ; /* Dummy size */
+	unsigned short	data	[2] ; /* Dummy size */
 } IMA_ADPCM_PRIVATE ;
 
 /*============================================================================================
@@ -60,10 +63,7 @@ static int ima_step_size [89] =
 static int ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;
 static int ima_writer_init (SF_PRIVATE *psf, int blockalign) ;
 
-static int ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
 static int ima_read_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima, short *ptr, int len) ;
-
-static int ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
 static int ima_write_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima, short *ptr, int len) ;
 
 static sf_count_t ima_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
@@ -79,7 +79,15 @@ static sf_count_t ima_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 static sf_count_t	ima_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset) ;
 
 static int	wav_w64_ima_close	(SF_PRIVATE *psf) ;
-/*-static int	aiff_ima_close	(SF_PRIVATE  *psf) ;-*/
+static int	aiff_ima_close	(SF_PRIVATE *psf) ;
+
+static int wav_w64_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
+static int wav_w64_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
+
+/*-static int aiff_ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;-*/
+static int aiff_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
+static int aiff_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
+
 
 /*============================================================================================
 ** IMA ADPCM Reader initialisation function.
@@ -120,7 +128,7 @@ wav_w64_ima_close	(SF_PRIVATE *psf)
 		**	as the final block.
 		*/
 		if (pima->samplecount && pima->samplecount < pima->samplesperblock)
-			ima_encode_block (psf, pima) ;
+			pima->encode_block (psf, pima) ;
 
 		psf->sf.frames = pima->samplesperblock * pima->blockcount / psf->sf.channels ;
 
@@ -128,72 +136,59 @@ wav_w64_ima_close	(SF_PRIVATE *psf)
 			psf->write_header (psf, SF_TRUE) ;
 		} ;
 
-	return 0 ;
-} /* wav_w64_ima_close */
-
-/*-
-int
-aiff_ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
-{	IMA_ADPCM_PUBLIC public ;
-
-	public.blocksize       = blockalign ;
-	public.channels        = psf->sf.channels ;
-	public.samplesperblock = samplesperblock ;
-	return ima_reader_init (psf, &public) ;
-} /+* aiff_ima_reader_init *+/
-
-int
-aiff_ima_writer_init (SF_PRIVATE *psf, int blocksize)
-{	IMA_ADPCM_PUBLIC 	public ;
-	int					error ;
-
-	public.blocksize = blocksize ;
-	public.channels  = psf->sf.channels ;
-	public.samplesperblock = 2 * (public.blocksize - 4 * psf->sf.channels) / psf->sf.channels + 1 ;
-
-	if ((error = ima_writer_init (psf, &public)))
-		return error ;
-
-	if (psf->write_header)
-		psf->write_header (psf) ;
-
-	psf->write_short  = ima_write_s ;
-	psf->write_int    = ima_write_i ;
-	psf->write_float  = ima_write_f ;
-	psf->write_double = ima_write_d ;
-	psf->seek    = ima_seek ;
-	psf->close        = aiff_ima_close ;
-
-	return 0 ;
-} /+* aiff_ima_writer_init *+/
-
-static int
-aiff_ima_close	(SF_PRIVATE  *psf)
-{	IMA_ADPCM_PRIVATE *pima ;
-
-	if (! psf->fdata)
-		return 0 ;
-
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
-
-	if (psf->mode == SFM_WRITE)
-	{	/+*	If a block has been partially assembled, write it out
-		**	as the final block.
-		*+/
-
-		if (pima->samplecount && pima->samplecount < pima->samplesperblock)
-			ima_encode_block (psf, pima) ;
-
-		if (psf->write_header)
-			psf->write_header (psf, SF_TRUE) ;
-		} ;
-
 	free (psf->fdata) ;
 	psf->fdata = NULL ;
 
 	return 0 ;
-} /+* aiff_ima_close *+/
--*/
+} /* wav_w64_ima_close */
+
+int
+aiff_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
+{	int error ;
+
+	if (psf->mode == SFM_RDWR)
+		return SFE_BAD_MODE_RW ;
+
+	if (psf->mode == SFM_READ)
+		if ((error = ima_reader_init (psf, blockalign, samplesperblock)))
+			return error ;
+
+	if (psf->mode == SFM_WRITE)
+		if ((error = ima_writer_init (psf, blockalign)))
+			return error ;
+
+	psf->seek = ima_seek ;
+	psf->close = aiff_ima_close ;
+
+	return 0 ;
+} /* aiff_ima_init */
+
+static int
+aiff_ima_close	(SF_PRIVATE *psf)
+{	IMA_ADPCM_PRIVATE *pima ;
+
+	if (! psf->fdata)
+		return 0 ;
+
+	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+
+	if (psf->mode == SFM_WRITE)
+	{	/*	If a block has been partially assembled, write it out
+		**	as the final block.
+		*/
+
+		if (pima->samplecount && pima->samplecount < pima->samplesperblock)
+			pima->encode_block (psf, pima) ;
+
+		if (psf->write_header)
+			psf->write_header (psf, SF_TRUE) ;
+		} ;
+
+	free (psf->fdata) ;
+	psf->fdata = NULL ;
+
+	return 0 ;
+} /* aiff_ima_close */
 
 /*============================================================================================
 ** IMA ADPCM Read Functions.
@@ -207,7 +202,7 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	if (psf->mode != SFM_READ)
 		return SFE_BAD_MODE_RW ;
 
-	pimasize = sizeof (IMA_ADPCM_PRIVATE) + blockalign + 3 * psf->sf.channels * samplesperblock ;
+	pimasize = sizeof (IMA_ADPCM_PRIVATE) + blockalign * psf->sf.channels + 3 * psf->sf.channels * samplesperblock ;
 
 	if (! (pima = malloc (pimasize)))
 		return SFE_MALLOC_FAILED ;
@@ -216,8 +211,8 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 
 	memset (pima, 0, pimasize) ;
 
-	pima->block		= (unsigned char*) pima->data ;
-	pima->samples	= (short*) (pima->data + blockalign) ;
+	pima->samples	= pima->data ;
+	pima->block		= (unsigned char*) (pima->data + samplesperblock * psf->sf.channels) ;
 
 	pima->channels			= psf->sf.channels ;
 	pima->blocksize			= blockalign ;
@@ -232,13 +227,32 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	else
 		pima->blocks = psf->datalength / pima->blocksize ;
 
-	count = 2 * (pima->blocksize - 4 * pima->channels) / pima->channels + 1 ;
-	if (pima->samplesperblock != count)
-		psf_log_printf (psf, "*** Warning : samplesperblock should be %d.\n", count) ;
+	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
+	{	case SF_FORMAT_WAV :
+		case SF_FORMAT_W64 :
+				count = 2 * (pima->blocksize - 4 * pima->channels) / pima->channels + 1 ;
 
-	psf->sf.frames = pima->samplesperblock * pima->blocks ;
+				if (pima->samplesperblock != count)
+					psf_log_printf (psf, "*** Warning : samplesperblock should be %d.\n", count) ;
 
-	ima_decode_block (psf, pima) ;	/* Read first block. */
+				pima->decode_block = wav_w64_ima_decode_block ;
+
+				psf->sf.frames = pima->samplesperblock * pima->blocks ;
+				break ;
+
+		case SF_FORMAT_AIFF :
+				psf_log_printf (psf, "still need to check block count\n") ;
+				pima->decode_block = aiff_ima_decode_block ;
+				psf->sf.frames = pima->samplesperblock * pima->blocks / pima->channels ;
+				break ;
+
+		default :
+				psf_log_printf (psf, "ima_reader_init: bad psf->sf.format\n") ;
+				return SFE_INTERNAL ;
+				break ;
+		} ;
+
+	pima->decode_block (psf, pima) ;	/* Read first block. */
 
 	psf->read_short		= ima_read_s ;
 	psf->read_int		= ima_read_i ;
@@ -249,7 +263,179 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 } /* ima_reader_init */
 
 static int
-ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
+aiff_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
+{	unsigned char *blockdata ;
+	int		chan, k, diff, bytecode ;
+	short	step, stepindx, predictor, *sampledata ;
+
+static int count = 0 ;
+count ++ ;
+
+	pima->blockcount += pima->channels ;
+	pima->samplecount = 0 ;
+
+	if (pima->blockcount > pima->blocks)
+	{	memset (pima->samples, 0, pima->samplesperblock * pima->channels * sizeof (short)) ;
+		return 1 ;
+		} ;
+
+	if ((k = psf_fread (pima->block, 1, pima->blocksize * pima->channels, psf)) != pima->blocksize * pima->channels)
+		psf_log_printf (psf, "*** Warning : short read (%d != %d).\n", k, pima->blocksize) ;
+
+	/* Read and check the block header. */
+	for (chan = 0 ; chan < pima->channels ; chan++)
+	{	blockdata = pima->block + chan * 34 ;
+		sampledata = pima->samples + chan ;
+
+		predictor = (blockdata [0] << 8) | (blockdata [1] & 0x80) ;
+		stepindx = blockdata [1] & 0x7F ;
+
+{
+if (count < 5)
+printf ("\nchan: %d    predictor: %d    stepindx: %d (%d)\n",
+	chan, predictor, stepindx, ima_step_size [stepindx]) ;
+}
+		/* FIXME : Do this a better way. */
+		if (stepindx < 0) stepindx = 0 ;
+		else if (stepindx > 88)	stepindx = 88 ;
+
+		/*
+		**	Pull apart the packed 4 bit samples and store them in their
+		**	correct sample positions.
+		*/
+		for (k = 0 ; k < pima->blocksize - 2 ; k++)
+		{	bytecode = blockdata [k + 2] ;
+			sampledata [pima->channels * (2 * k + 0)] = bytecode & 0xF ;
+			sampledata [pima->channels * (2 * k + 1)] = (bytecode >> 4) & 0xF ;
+			} ;
+
+		/* Decode the encoded 4 bit samples. */
+		for (k = 0 ; k < pima->samplesperblock ; k ++)
+		{	step = ima_step_size [stepindx] ;
+
+			bytecode = pima->samples [pima->channels * k + chan] ;
+
+			stepindx += ima_indx_adjust [bytecode] ;
+
+			if (stepindx < 0) stepindx = 0 ;
+			else if (stepindx > 88) stepindx = 88 ;
+
+			diff = step >> 3 ;
+			if (bytecode & 1)	diff += step >> 2 ;
+			if (bytecode & 2)	diff += step >> 1 ;
+			if (bytecode & 4)	diff += step ;
+			if (bytecode & 8)	diff = -diff ;
+
+			predictor += diff ;
+
+			pima->samples [pima->channels * k + chan] = predictor ;
+			} ;
+		} ;
+
+if (count < 5)
+{
+	for (k = 0 ; k < 10 ; k++)
+		printf ("% 7d,", pima->samples [k]) ;
+	puts ("") ;
+}
+
+	return 1 ;
+} /* aiff_ima_decode_block */
+
+static int
+aiff_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
+{	int		chan, k, step, diff, vpdiff, blockindx, indx ;
+	short	bytecode, mask ;
+
+static int count = 0 ;
+if (0 && count == 0)
+{	pima->samples [0] = 0 ;
+	printf ("blocksize : %d\n", pima->blocksize) ;
+	printf ("pima->stepindx [0] : %d\n", pima->stepindx [0]) ;
+	}
+count ++ ;
+
+	/* Encode the block header. */
+	for (chan = 0 ; chan < pima->channels ; chan ++)
+	{	blockindx = chan * pima->blocksize ;
+
+		pima->block [blockindx] = (pima->samples [chan] >> 8) & 0xFF ;
+		pima->block [blockindx + 1] = (pima->samples [chan] & 0x80) + (pima->stepindx [chan] & 0x7F) ;
+		
+		pima->previous [chan] = pima->samples [chan] ;
+		} ;
+
+	/* Encode second and later samples for every block as a 4 bit value. */
+	for (k = pima->channels ; k < (pima->samplesperblock * pima->channels) ; k ++)
+	{	chan = (pima->channels > 1) ? (k % 2) : 0 ;
+
+		diff = pima->samples [k] - pima->previous [chan] ;
+
+		bytecode = 0 ;
+		step = ima_step_size [pima->stepindx [chan]] ;
+		vpdiff = step >> 3 ;
+		if (diff < 0)
+		{	bytecode = 8 ;
+			diff = -diff ;
+			} ;
+		mask = 4 ;
+		while (mask)
+		{	if (diff >= step)
+			{	bytecode |= mask ;
+				diff -= step ;
+				vpdiff += step ;
+				} ;
+			step >>= 1 ;
+			mask >>= 1 ;
+			} ;
+
+		if (bytecode & 8)
+			pima->previous [chan] -= vpdiff ;
+		else
+			pima->previous [chan] += vpdiff ;
+
+		if (pima->previous [chan] > 32767)
+			pima->previous [chan] = 32767 ;
+		else if (pima->previous [chan] < -32768)
+			pima->previous [chan] = -32768 ;
+
+		pima->stepindx [chan] += ima_indx_adjust [bytecode] ;
+		if (pima->stepindx [chan] < 0)
+			pima->stepindx [chan] = 0 ;
+		else if (pima->stepindx [chan] > 88)
+			pima->stepindx [chan] = 88 ;
+
+		pima->samples [k] = bytecode ;
+		} ;
+
+	/* Pack the 4 bit encoded samples. */
+
+	for (chan = 0 ; chan < pima->channels ; chan ++)
+	{	for (indx = pima->channels ; indx < pima->channels * pima->samplesperblock ; indx += 2 * pima->channels)
+		{	blockindx = chan * pima->blocksize + 2  + indx / 2;
+
+if (0 && count ++ < 5)
+	printf ("chan: %d    blockindx: %3d    indx: %3d\n", chan, blockindx, indx) ;
+		
+			pima->block [blockindx] = pima->samples [indx] & 0x0F ;
+			pima->block [blockindx] |= (pima->samples [indx + pima->channels] << 4) & 0xF0 ;
+			} ;
+		} ;
+
+	/* Write the block to disk. */
+
+	if ((k = psf_fwrite (pima->block, 1, pima->channels * pima->blocksize, psf)) != pima->channels * pima->blocksize)
+		psf_log_printf (psf, "*** Warning : short write (%d != %d).\n", k, pima->channels * pima->blocksize) ;
+
+	memset (pima->samples, 0, pima->channels * pima->samplesperblock * sizeof (short)) ;
+	pima->samplecount = 0 ;
+	pima->blockcount ++ ;
+
+	return 1 ;
+} /* aiff_ima_encode_block */
+
+static int
+wav_w64_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 {	int		chan, k, current, blockindx, indx, indxstart, diff ;
 	short	step, bytecode, stepindx [2] ;
 
@@ -281,14 +467,11 @@ ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 			psf_log_printf (psf, "IMA ADPCM synchronisation error.\n") ;
 
 		pima->samples [chan] = current ;
-
-		/* psf_log_printf (psf, "block %d : channel %d (current, indx) : (%d, %d)\n",
-		**	pima->blockcount, chan,  current, stepindx [chan]) ;
-		*/
 		} ;
 
-	/* Pull apart the packed 4 bit samples and store them in their
-	** correct sample positions.
+	/*
+	**	Pull apart the packed 4 bit samples and store them in their
+	**	correct sample positions.
 	*/
 
 	blockindx = 4 * pima->channels ;
@@ -346,7 +529,98 @@ ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 		} ;
 
 	return 1 ;
-} /* ima_decode_block */
+} /* wav_w64_ima_decode_block */
+
+static int
+wav_w64_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
+{	int		chan, k, step, diff, vpdiff, blockindx, indx, indxstart ;
+	short	bytecode, mask ;
+
+	/* Encode the block header. */
+	for (chan = 0 ; chan < pima->channels ; chan++)
+	{	pima->block [chan*4]	= pima->samples [chan] & 0xFF ;
+		pima->block [chan*4+1]	= (pima->samples [chan] >> 8) & 0xFF ;
+
+		pima->block [chan*4+2] = pima->stepindx [chan] ;
+		pima->block [chan*4+3] = 0 ;
+
+		pima->previous [chan] = pima->samples [chan] ;
+		} ;
+
+	/* Encode the samples as 4 bit. */
+
+	for (k = pima->channels ; k < (pima->samplesperblock * pima->channels) ; k ++)
+	{	chan = (pima->channels > 1) ? (k % 2) : 0 ;
+
+		diff = pima->samples [k] - pima->previous [chan] ;
+
+		bytecode = 0 ;
+		step = ima_step_size [pima->stepindx [chan]] ;
+		vpdiff = step >> 3 ;
+		if (diff < 0)
+		{	bytecode = 8 ;
+			diff = -diff ;
+			} ;
+		mask = 4 ;
+		while (mask)
+		{	if (diff >= step)
+			{	bytecode |= mask ;
+				diff -= step ;
+				vpdiff += step ;
+				} ;
+			step >>= 1 ;
+			mask >>= 1 ;
+			} ;
+
+		if (bytecode & 8)
+			pima->previous [chan] -= vpdiff ;
+		else
+			pima->previous [chan] += vpdiff ;
+
+		if (pima->previous [chan] > 32767)
+			pima->previous [chan] = 32767 ;
+		else if (pima->previous [chan] < -32768)
+			pima->previous [chan] = -32768 ;
+
+		pima->stepindx [chan] += ima_indx_adjust [bytecode] ;
+		if (pima->stepindx [chan] < 0)
+			pima->stepindx [chan] = 0 ;
+		else if (pima->stepindx [chan] > 88)
+			pima->stepindx [chan] = 88 ;
+
+		pima->samples [k] = bytecode ;
+		} ;
+
+	/* Pack the 4 bit encoded samples. */
+
+	blockindx = 4 * pima->channels ;
+
+	indxstart = pima->channels ;
+	while (blockindx < pima->blocksize)
+	{	for (chan = 0 ; chan < pima->channels ; chan++)
+		{	indx = indxstart + chan ;
+			for (k = 0 ; k < 4 ; k++)
+			{	pima->block [blockindx] = pima->samples [indx] & 0x0F ;
+				indx += pima->channels ;
+				pima->block [blockindx] |= (pima->samples [indx] << 4) & 0xF0 ;
+				indx += pima->channels ;
+				blockindx ++ ;
+				} ;
+			} ;
+		indxstart += 8 * pima->channels ;
+		} ;
+
+	/* Write the block to disk. */
+
+	if ((k = psf_fwrite (pima->block, 1, pima->blocksize, psf)) != pima->blocksize)
+		psf_log_printf (psf, "*** Warning : short write (%d != %d).\n", k, pima->blocksize) ;
+
+	memset (pima->samples, 0, pima->samplesperblock * sizeof (short)) ;
+	pima->samplecount = 0 ;
+	pima->blockcount ++ ;
+
+	return 1 ;
+} /* wav_w64_ima_encode_block */
 
 static int
 ima_read_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima, short *ptr, int len)
@@ -359,7 +633,7 @@ ima_read_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima, short *ptr, int len)
 			} ;
 
 		if (pima->samplecount >= pima->samplesperblock)
-			ima_decode_block (psf, pima) ;
+			pima->decode_block (psf, pima) ;
 
 		count = (pima->samplesperblock - pima->samplecount) * pima->channels ;
 		count = (len - indx > count) ? count : len - indx ;
@@ -501,7 +775,7 @@ ima_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 	if (offset == 0)
 	{	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
 		pima->blockcount = 0 ;
-		ima_decode_block (psf, pima) ;
+		pima->decode_block (psf, pima) ;
 		pima->samplecount = 0 ;
 		return 0 ;
 		} ;
@@ -517,7 +791,7 @@ ima_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 	if (mode == SFM_READ)
 	{	psf_fseek (psf, psf->dataoffset + newblock * pima->blocksize, SEEK_SET) ;
 		pima->blockcount = newblock ;
-		ima_decode_block (psf, pima) ;
+		pima->decode_block (psf, pima) ;
 		pima->samplecount = newsample ;
 		}
 	else
@@ -546,13 +820,11 @@ ima_writer_init (SF_PRIVATE *psf, int blockalign)
 
 	pimasize = sizeof (IMA_ADPCM_PRIVATE) + blockalign + 3 * psf->sf.channels * samplesperblock ;
 
-	if (! (pima = malloc (pimasize)))
+	if ((pima = calloc (1, pimasize)) == NULL)
 		return SFE_MALLOC_FAILED ;
 
 	psf->fdata = (void*) pima ;
 
-	memset (pima, 0, pimasize) ;
-
 	pima->channels			= psf->sf.channels ;
 	pima->blocksize			= blockalign ;
 	pima->samplesperblock	= samplesperblock ;
@@ -562,6 +834,22 @@ ima_writer_init (SF_PRIVATE *psf, int blockalign)
 
 	pima->samplecount = 0 ;
 
+	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
+	{	case SF_FORMAT_WAV :
+		case SF_FORMAT_W64 :
+				pima->encode_block = wav_w64_ima_encode_block ;
+				break ;
+
+		case SF_FORMAT_AIFF :
+				pima->encode_block = aiff_ima_encode_block ;
+				break ;
+
+		default :
+				psf_log_printf (psf, "ima_reader_init: bad psf->sf.format\n") ;
+				return SFE_INTERNAL ;
+				break ;
+		} ;
+
 	psf->write_short	= ima_write_s ;
 	psf->write_int		= ima_write_i ;
 	psf->write_float	= ima_write_f ;
@@ -570,103 +858,9 @@ ima_writer_init (SF_PRIVATE *psf, int blockalign)
 	return 0 ;
 } /* ima_writer_init */
 
-
-
 /*==========================================================================================
 */
 
-static int
-ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
-{	int		chan, k, step, diff, vpdiff, blockindx, indx, indxstart ;
-	short	bytecode, mask ;
-
-	/* Encode the block header. */
-	for (chan = 0 ; chan < pima->channels ; chan++)
-	{	pima->block [chan*4]	= pima->samples [chan] & 0xFF ;
-		pima->block [chan*4+1]	= (pima->samples [chan] >> 8) & 0xFF ;
-
-		pima->block [chan*4+2] = pima->stepindx [chan] ;
-		pima->block [chan*4+3] = 0 ;
-		} ;
-
-	pima->previous [0] = pima->samples [0] ;
-	pima->previous [1] = pima->samples [1] ;
-
-	/* Encode the samples as 4 bit. */
-
-	for (k = pima->channels ; k < (pima->samplesperblock * pima->channels) ; k ++)
-	{	chan = (pima->channels > 1) ? (k % 2) : 0 ;
-
-		diff = pima->samples [k] - pima->previous [chan] ;
-
-		bytecode = 0 ;
-		step = ima_step_size [pima->stepindx [chan]] ;
-		vpdiff = step >> 3 ;
-		if (diff < 0)
-		{	bytecode = 8 ;
-			diff = -diff ;
-			} ;
-		mask = 4 ;
-		while (mask)
-		{	if (diff >= step)
-			{	bytecode |= mask ;
-				diff -= step ;
-				vpdiff += step ;
-				} ;
-			step >>= 1 ;
-			mask >>= 1 ;
-			} ;
-
-		if (bytecode & 8)
-			pima->previous [chan] -= vpdiff ;
-		else
-			pima->previous [chan] += vpdiff ;
-
-		if (pima->previous [chan] > 32767)
-			pima->previous [chan] = 32767 ;
-		else if (pima->previous [chan] < -32768)
-			pima->previous [chan] = -32768 ;
-
-		pima->stepindx [chan] += ima_indx_adjust [bytecode] ;
-		if (pima->stepindx [chan] < 0)
-			pima->stepindx [chan] = 0 ;
-		else if (pima->stepindx [chan] > 88)
-			pima->stepindx [chan] = 88 ;
-
-		pima->samples [k] = bytecode ;
-		} ;
-
-	/* Pack the 4 bit encoded samples. */
-
-	blockindx = 4 * pima->channels ;
-
-	indxstart = pima->channels ;
-	while (blockindx < pima->blocksize)
-	{	for (chan = 0 ; chan < pima->channels ; chan++)
-		{	indx = indxstart + chan ;
-			for (k = 0 ; k < 4 ; k++)
-			{	pima->block [blockindx] = pima->samples [indx] & 0x0F ;
-				indx += pima->channels ;
-				pima->block [blockindx] |= (pima->samples [indx] << 4) & 0xF0 ;
-				indx += pima->channels ;
-				blockindx ++ ;
-				} ;
-			} ;
-		indxstart += 8 * pima->channels ;
-		} ;
-
-	/* Write the block to disk. */
-
-	if ((k = psf_fwrite (pima->block, 1, pima->blocksize, psf)) != pima->blocksize)
-		psf_log_printf (psf, "*** Warning : short write (%d != %d).\n", k, pima->blocksize) ;
-
-	memset (pima->samples, 0, pima->samplesperblock * sizeof (short)) ;
-	pima->samplecount = 0 ;
-	pima->blockcount ++ ;
-
-	return 1 ;
-} /* ima_encode_block */
-
 static int
 ima_write_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima, short *ptr, int len)
 {	int		count, total = 0, indx = 0 ;
@@ -683,7 +877,7 @@ ima_write_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima, short *ptr, int len)
 		total = indx ;
 
 		if (pima->samplecount >= pima->samplesperblock)
-			ima_encode_block (psf, pima) ;
+			pima->encode_block (psf, pima) ;
 		} ;
 
 	return total ;
