@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -230,11 +230,11 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 
 					if (psf->fileoffset > 0 && psf->filelength > RIFFsize + 8)
 					{	/* Set file length. */
-						psf->filelength = RIFFsize + 8  ;
+						psf->filelength = RIFFsize + 8 ;
 						psf_log_printf (psf, "RIFF : %u\n", RIFFsize) ;
 						}
 					else if (psf->filelength < RIFFsize + 2 * SIGNED_SIZEOF (dword))
-					{	psf_log_printf (psf, "RIFF : %u (should be %D)\n", (sf_count_t) RIFFsize, psf->filelength - 2 * SIGNED_SIZEOF (dword)) ;
+					{	psf_log_printf (psf, "RIFF : %u (should be %D)\n", RIFFsize, psf->filelength - 2 * SIGNED_SIZEOF (dword)) ;
 						RIFFsize = dword ;
 						}
 					else if (RIFFsize < 100)
@@ -476,12 +476,12 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 
 	if (psf->is_pipe == 0)
 	{	/*
-	** Check for 'wvpk' at the start of the DATA section. Not able to
-	** handle this.
-	*/
-	psf_binheader_readf (psf, "e4", &marker) ;
+		** Check for 'wvpk' at the start of the DATA section. Not able to
+		** handle this.
+		*/
+		psf_binheader_readf (psf, "e4", &marker) ;
 		if (marker == wvpk_MARKER || marker == OggS_MARKER)
-		return SFE_WAV_WVPK_DATA ;
+			return SFE_WAV_WVPK_DATA ;
 		} ;
 
 	/* Seek to start of DATA section. */
